@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1997-2023, University of Amsterdam
+    Copyright (c)  1997-2024, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
                               SWI-Prolog Solutions b.v.
@@ -214,6 +214,9 @@ iso_message(existence_error(matching_rule, Goal)) -->
     [ 'No rule matches ~p'-[Goal] ].
 iso_message(existence_error(Type, Object)) -->
     [ '~w `~p'' does not exist'-[Type, Object] ].
+iso_message(existence_error(export, PI, module(M))) --> % not ISO
+    [ 'Module ', ansi(code, '~q', [M]), ' does not export ',
+      ansi(code, '~q', [PI]) ].
 iso_message(existence_error(Type, Object, In)) --> % not ISO
     [ '~w `~p'' does not exist in ~p'-[Type, Object, In] ].
 iso_message(busy(Type, Object)) -->
@@ -313,7 +316,7 @@ dependency_error(Dep, monotonic(On)) -->
     ].
 
 faq(Page) -->
-    [nl, '  See FAQ at https://www.swi-prolog.org/FAQ/', Page, '.txt' ].
+    [nl, '  See FAQ at https://www.swi-prolog.org/FAQ/', Page, '.html' ].
 
 type_error_comment(_Expected, Actual) -->
     { type_of(Actual, Type),
@@ -1317,6 +1320,7 @@ query_result(help) -->
       '  ; (n,r,space,TAB): redo              | t:         trace&redo'-[], nl,
       '  *:                 show choicepoint  | c (a,RET): stop'-[], nl,
       '  w:                 write             | p:         print'-[], nl,
+      '  +:                 max_depth*10      | -:         max_depth//10'-[], nl,
       '  b:                 break             | h (?):     help'-[],
       nl, nl
     ].
@@ -1574,6 +1578,12 @@ prolog_message(spying(Heads)) -->
 prolog_message(trace(Head, [])) -->
     !,
     [ '    ' ], goal_predicate(Head), [ ' Not tracing'-[], nl].
+prolog_message(trace(Head, Ports)) -->
+    { '$member'(Port, Ports), compound(Port),
+      !,
+      numbervars(Head+Ports, 0, _, [singletons(true)])
+    },
+    [ '    ~p: ~p'-[Head,Ports] ].
 prolog_message(trace(Head, Ports)) -->
     [ '    ' ], goal_predicate(Head), [ ': ~w'-[Ports], nl].
 prolog_message(tracing([])) -->
@@ -1838,6 +1848,10 @@ target_module(Module) -->
 		 *           TRIPWIRES		*
 		 *******************************/
 
+tripwire_message(max_integer_size, Bytes) -->
+    !,
+    [ 'Trapped tripwire max_integer_size: big integers and \c
+       rationals are limited to ~D bytes'-[Bytes] ].
 tripwire_message(Wire, Context) -->
     [ 'Trapped tripwire ~w for '-[Wire] ],
     tripwire_context(Wire, Context).
@@ -1969,6 +1983,9 @@ default_theme(message(Level),         Attrs) :-
 :- thread_local
     user:thread_message_hook/3.
 :- '$hide'((push_msg/1,pop_msg/0)).
+:- '$notransact'((user:message_hook/3,
+                  prolog:message_prefix_hook/2,
+                  user:thread_message_hook/3)).
 
 %!  print_message(+Kind, +Term)
 %
