@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1997-2021, University of Amsterdam
+    Copyright (c)  1997-2024, University of Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -109,6 +109,8 @@ typedef enum
 #define	PL_get_size_ex(t, i)		LDFUNC(PL_get_size_ex, t, i)
 #define	pl_get_uint64(t, i, ex)		LDFUNC(pl_get_uint64, t, i, ex)
 #define	PL_get_uint64_ex(t, i)		LDFUNC(PL_get_uint64_ex, t, i)
+#define PL_get_long_ex(t, i)		LDFUNC(PL_get_long_ex, t, i)
+#define PL_get_intptr_ex(t, i)		LDFUNC(PL_get_intptr_ex, t, i)
 #endif /*USE_LD_MACROS*/
 
 #define LDFUNC_DECLARATIONS
