@@ -100,7 +100,7 @@ option  parsing,  initialisation  and  handling  of errors and warnings.
 
 static int	usage(void);
 static int	giveVersionInfo(const char *a);
-static bool	vsysError(const char *fm, va_list args);
+static bool	vsysError(const char *errtype, const char *fm, va_list args);
 
 #define	optionString(s) { if (argc > 1) \
 			  { if ( s ) remove_string(s); \
@@ -1734,13 +1734,12 @@ sysError(const char *fm, ...)
 { va_list args;
 
   va_start(args, fm);
-  vsysError(fm, args);
+  vsysError("system", fm, args);
   va_end(args);
 
   PL_fail;
 }
 
-
 void
 fatalError(const char *fm, ...)
 { va_list args;
@@ -1777,7 +1776,9 @@ printCrashContext(const char *btname)
   static bool running = FALSE;
 
   if ( running )
-    Sdprintf("Recursive crash; omitting crash report\n");
+  { Sdprintf("Recursive crash; omitting crash report\n");
+    return;
+  }
   running = TRUE;
 
   now = time(NULL);
@@ -1785,7 +1786,9 @@ printCrashContext(const char *btname)
   tbuf[24] = '\0';
   Sdprintf("Time: %s\n", tbuf);
 
+#ifdef O_PLMT
   if ( LD )
+#endif
   { Sdprintf("Inferences: %" PRIu64 "\n", LD->statistics.inferences);
     if ( gc_status.active )
     { Sdprintf("Running GC: %" PRIu64 "-th garbage collection]\n",
@@ -1807,7 +1810,9 @@ printCrashContext(const char *btname)
 #endif
 
   print_backtrace_named(btname);
+#ifdef O_PLMT
   if ( LD )
+#endif
   { if ( getenv("SWIPL_DEBUG_GC_STACK") )
     { if ( LD->shift_status.inferences )
       { Sdprintf("Last stack shift at %" PRIu64 " inferences\n",
@@ -1827,21 +1832,44 @@ printCrashContext(const char *btname)
   running = FALSE;
 }
 
+#ifdef HAVE_SETITIMER
+static void
+abort_sig(int sig)
+{ (void)sig;
+  abort();
+}
+#endif
 
+static void
+set_cleanup_timeout(int sec)
+{
+#ifdef HAVE_SETITIMER
+  struct itimerval timeout = {0};
+  struct sigaction act = {0};
+
+  timeout.it_value.tv_sec = sec;
+  act.sa_handler = abort_sig;
+
+  sigaction(SIGALRM, &act, NULL);
+  setitimer(ITIMER_REAL, &timeout, NULL);
+#endif
+}
 
 static bool
-vsysError(const char *fm, va_list args)
+vsysError(const char *errtype, const char *fm, va_list args)
 { static int active = 0;
 
   if ( active++ )
     abort();
 
-  Sfprintf(Serror, "\nERROR: System error: ");
+  set_cleanup_timeout(30);
+
+  Sfprintf(Serror, "\nERROR: %s error: ", errtype);
   Svfprintf(Serror, fm, args);
   Sfprintf(Serror, "\n");
 
-  save_backtrace("SYSERROR");
-  printCrashContext("SYSERROR");
+  save_backtrace(errtype);
+  printCrashContext(errtype);
 
   if ( !(true(Sinput, SIO_ISATTY) &&
 	 true(Serror, SIO_ISATTY)) ||
@@ -1849,13 +1877,13 @@ vsysError(const char *fm, va_list args)
     PL_abort_process();			/* non-interactive or booting */
 
 action:
+  Sflush(Soutput);
+  ResetTty();
 #ifdef HAVE_GETPID
   Sfprintf(Serror, "\n[pid=%d] Action? ", getpid());
 #else
   Sfprintf(Serror, "\nAction? ");
 #endif
-  Sflush(Soutput);
-  ResetTty();
 
   switch(getSingleChar(Sinput, FALSE))
   { case EOF:
@@ -1878,7 +1906,16 @@ PL_system_error(const char *fm, ...)
 { va_list args;
 
   va_start(args, fm);
-  vsysError(fm, args);
+  vsysError("system", fm, args);
+  va_end(args);
+}
+
+void
+PL_api_error(const char *fm, ...)
+{ va_list args;
+
+  va_start(args, fm);
+  vsysError("API", fm, args);
   va_end(args);
 }
 
